# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         chat_api.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/01/02
# -------------------------------------------------------------------------------
import base64
import json
import logging
import os
import re
import time
import shutil
import pythoncom
import pywxdump

from flask import Flask, request, render_template, g, Blueprint, send_file, make_response, session
from pywxdump import get_core_db, all_merge_real_time_db
from pywxdump.api.rjson import ReJson, RqJson
from pywxdump.api.utils import get_conf, get_conf_wxids, set_conf, error9999, gen_base64, validate_title, \
    get_conf_local_wxid
from pywxdump import get_wx_info, WX_OFFS, batch_decrypt, BiasAddr, merge_db, decrypt_merge, merge_real_time_db

from pywxdump.db import DBHandler, download_file, export_csv, export_json, dat2img

# app = Flask(__name__, static_folder='../ui/web/dist', static_url_path='/')

rs_api = Blueprint('rs_api', __name__, template_folder='../ui/web', static_folder='../ui/web/assets/', )
rs_api.debug = False


# 是否初始化
@rs_api.route('/api/rs/is_init', methods=["GET", 'POST'])
@error9999
def is_init():
    """
    是否初始化
    :return:
    """
    local_wxids = get_conf_local_wxid(g.caf)
    if len(local_wxids) > 1:
        return ReJson(0, True)
    return ReJson(0, False)


# start 以下为聊天联系人相关api *******************************************************************************************

@rs_api.route('/api/rs/mywxid', methods=["GET", 'POST'])
@error9999
def mywxid():
    """
    获取我的微信id
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    return ReJson(0, {"my_wxid": my_wxid})


@rs_api.route('/api/rs/user_session_list', methods=["GET", 'POST'])
@error9999
def user_session_list():
    """
    获取联系人列表
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")
    db = DBHandler(db_config)
    ret = db.get_session_list()
    return ReJson(0, list(ret.values()))


@rs_api.route('/api/rs/user_labels_dict', methods=["GET", 'POST'])
@error9999
def user_labels_dict():
    """
    获取标签字典
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")
    db = DBHandler(db_config)
    user_labels_dict = db.get_labels()
    return ReJson(0, user_labels_dict)


@rs_api.route('/api/rs/user_list', methods=["GET", 'POST'])
@error9999
def user_list():
    """
    获取联系人列表，可用于搜索
    :return:
    """
    if request.method == "GET":
        word = request.args.get("word", "")
        wxids = request.args.get("wxids", [])
        labels = request.args.get("labels", [])
    elif request.method == "POST":
        word = request.json.get("word", "")
        wxids = request.json.get("wxids", [])
        labels = request.json.get("labels", [])
    else:
        return ReJson(1003, msg="Unsupported method")

    if isinstance(wxids, str) and wxids == '' or wxids is None: wxids = []
    if isinstance(labels, str) and labels == '' or labels is None: labels = []

    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")
    db = DBHandler(db_config)
    users = db.get_user(word, wxids, labels)
    return ReJson(0, users)


# end 以上为聊天联系人相关api *********************************************************************************************

# start 以下为聊天记录相关api *********************************************************************************************


@rs_api.route('/api/rs/imgsrc/<path:imgsrc>', methods=["GET", 'POST'])
@error9999
def get_imgsrc(imgsrc):
    """
    获取图片,
    1. 从网络获取图片，主要功能只是下载图片，缓存到本地
    2. 读取本地图片
    :return:
    """
    if not imgsrc:
        return ReJson(1002)
    if imgsrc.startswith("FileStorage"):  # 如果是本地图片文件则调用get_img
        my_wxid = get_conf(g.caf, g.at, "last")
        if not my_wxid: return ReJson(1001, body="my_wxid is required")
        wx_path = get_conf(g.caf, my_wxid, "wx_path")

        img_path = imgsrc.replace("\\\\", "\\")

        img_tmp_path = os.path.join(g.work_path, my_wxid, "img")
        original_img_path = os.path.join(wx_path, img_path)
        if os.path.exists(original_img_path):
            rc, fomt, md5, out_bytes = dat2img(original_img_path)
            if not rc:
                return ReJson(1001, body=original_img_path)
            imgsavepath = os.path.join(str(img_tmp_path), img_path + "_" + "".join([md5, fomt]))
            if os.path.exists(imgsavepath):
                return send_file(imgsavepath)
            if not os.path.exists(os.path.dirname(imgsavepath)):
                os.makedirs(os.path.dirname(imgsavepath))
            with open(imgsavepath, "wb") as f:
                f.write(out_bytes)
            return send_file(imgsavepath)
        else:
            return ReJson(1001, body=f"{original_img_path} not exists")
    elif imgsrc.startswith("http://") or imgsrc.startswith("https://"):
        # 将?后面的参数连接到imgsrc
        imgsrc = imgsrc + "?" + request.query_string.decode("utf-8") if request.query_string else imgsrc
        my_wxid = get_conf(g.caf, g.at, "last")
        if not my_wxid: return ReJson(1001, body="my_wxid is required")

        img_tmp_path = os.path.join(g.work_path, my_wxid, "imgsrc")
        if not os.path.exists(img_tmp_path):
            os.makedirs(img_tmp_path)
        file_name = imgsrc.replace("http://", "").replace("https://", "").replace("/", "_").replace("?", "_")
        file_name = file_name + ".jpg"
        # 如果文件名过长，则将文件明分为目录和文件名
        if len(file_name) > 255:
            file_name = file_name[:255] + "/" + file_name[255:]

        img_path_all = os.path.join(str(img_tmp_path), file_name)
        if os.path.exists(img_path_all):
            return send_file(img_path_all)
        else:
            download_file(imgsrc, img_path_all)
            if os.path.exists(img_path_all):
                return send_file(img_path_all)
            else:
                return ReJson(4004, body=imgsrc)
    else:
        return ReJson(1002, body=imgsrc)


@rs_api.route('/api/rs/msg_count', methods=["GET", 'POST'])
@error9999
def msg_count():
    """
    获取联系人的聊天记录数量
    :return:
    """
    if request.method == "GET":
        wxid = request.args.get("wxids", [])
    elif request.method == "POST":
        wxid = request.json.get("wxids", [])
    else:
        return ReJson(1003, msg="Unsupported method")

    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")
    db = DBHandler(db_config)
    chat_count = db.get_msg_count(wxid)
    chat_count1 = db.get_plc_msg_count(wxid)
    # 合并两个字典，相同key，则将value相加
    count = {k: chat_count.get(k, 0) + chat_count1.get(k, 0) for k in
             list(set(list(chat_count.keys()) + list(chat_count1.keys())))}
    return ReJson(0, count)


@rs_api.route('/api/rs/msg_list', methods=["GET", 'POST'])
@error9999
def get_msgs():
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")

    start = request.json.get("start")
    limit = request.json.get("limit")
    wxid = request.json.get("wxid")

    if not wxid:
        return ReJson(1002, body=f"wxid is required: {wxid}")
    if start and isinstance(start, str) and start.isdigit():
        start = int(start)
    if limit and isinstance(limit, str) and limit.isdigit():
        limit = int(limit)
    if start is None or limit is None:
        return ReJson(1002, body=f"start or limit is required {start} {limit}")
    if not isinstance(start, int) and not isinstance(limit, int):
        return ReJson(1002, body=f"start or limit is not int {start} {limit}")

    db = DBHandler(db_config)
    msgs, wxid_list = db.get_msg_list(wxid=wxid, start_index=start, page_size=limit)
    if not msgs:
        msgs, wxid_list = db.get_plc_msg_list(wxid=wxid, start_index=start, page_size=limit)
    wxid_list.append(my_wxid)
    user = db.get_user_list(wxids=wxid_list)
    return ReJson(0, {"msg_list": msgs, "user_list": user})


@rs_api.route('/api/rs/video/<path:videoPath>', methods=["GET", 'POST'])
def get_video(videoPath):
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = get_conf(g.caf, my_wxid, "wx_path")

    videoPath = videoPath.replace("\\\\", "\\")

    video_tmp_path = os.path.join(g.work_path, my_wxid, "video")
    original_img_path = os.path.join(wx_path, videoPath)
    if not os.path.exists(original_img_path):
        return ReJson(5002)
    # 复制文件到临时文件夹
    video_save_path = os.path.join(video_tmp_path, videoPath)
    if not os.path.exists(os.path.dirname(video_save_path)):
        os.makedirs(os.path.dirname(video_save_path))
    if os.path.exists(video_save_path):
        return send_file(video_save_path)
    shutil.copy(original_img_path, video_save_path)
    return send_file(original_img_path)


@rs_api.route('/api/rs/audio/<path:savePath>', methods=["GET", 'POST'])
def get_audio(savePath):
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")

    savePath = os.path.join(g.work_path, my_wxid, "audio", savePath)  # 这个是从url中获取的
    if os.path.exists(savePath):
        return send_file(savePath)

    MsgSvrID = savePath.split("_")[-1].replace(".wav", "")
    if not savePath:
        return ReJson(1002)

    # 判断savePath路径的文件夹是否存在
    if not os.path.exists(os.path.dirname(savePath)):
        os.makedirs(os.path.dirname(savePath))

    db = DBHandler(db_config)
    wave_data = db.get_audio(MsgSvrID, is_play=False, is_wave=True, save_path=savePath, rate=24000)
    if not wave_data:
        return ReJson(1001, body="wave_data is required")

    if os.path.exists(savePath):
        return send_file(savePath)
    else:
        return ReJson(4004, body=savePath)


@rs_api.route('/api/rs/file_info', methods=["GET", 'POST'])
def get_file_info():
    file_path = request.args.get("file_path")
    file_path = request.json.get("file_path", file_path)
    if not file_path:
        return ReJson(1002)

    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = get_conf(g.caf, my_wxid, "wx_path")

    all_file_path = os.path.join(wx_path, file_path)
    if not os.path.exists(all_file_path):
        return ReJson(5002)
    file_name = os.path.basename(all_file_path)
    file_size = os.path.getsize(all_file_path)
    return ReJson(0, {"file_name": file_name, "file_size": str(file_size)})


@rs_api.route('/api/rs/file/<path:filePath>', methods=["GET", 'POST'])
def get_file(filePath):
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = get_conf(g.caf, my_wxid, "wx_path")

    all_file_path = os.path.join(wx_path, filePath)
    if not os.path.exists(all_file_path):
        return ReJson(5002)
    return send_file(all_file_path)


# end 以上为聊天记录相关api *********************************************************************************************

# start 导出聊天记录 *****************************************************************************************************

@rs_api.route('/api/rs/export_endb', methods=["GET", 'POST'])
def get_export_endb():
    """
    导出加密数据库
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = get_conf(g.caf, my_wxid, "wx_path")
    wx_path = request.json.get("wx_path", wx_path)

    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")

    # 分割wx_path的文件名和父目录
    code, wxdbpaths = get_core_db(wx_path)
    if not code:
        return ReJson(2001, body=wxdbpaths)

    outpath = os.path.join(g.work_path, "export", my_wxid, "endb")
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    for wxdb in wxdbpaths:
        # 复制wxdb->outpath, os.path.basename(wxdb)
        assert isinstance(outpath, str)  # 为了解决pycharm的警告, 无实际意义
        shutil.copy(wxdb, os.path.join(outpath, os.path.basename(wxdb)))
    return ReJson(0, body=outpath)


@rs_api.route('/api/rs/export_dedb', methods=["GET", "POST"])
def get_export_dedb():
    """
    导出解密数据库
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    key = request.json.get("key", get_conf(g.caf, my_wxid, "key"))
    wx_path = request.json.get("wx_path", get_conf(g.caf, my_wxid, "wx_path"))

    if not key:
        return ReJson(1002, body=f"key is required: {key}")
    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")

    outpath = os.path.join(g.work_path, "export", my_wxid, "dedb")
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, merge_save_path = decrypt_merge(wx_path=wx_path, key=key, outpath=outpath)
    time.sleep(1)
    if code:
        return ReJson(0, body=merge_save_path)
    else:
        return ReJson(2001, body=merge_save_path)


@rs_api.route('/api/rs/export_csv', methods=["GET", 'POST'])
def get_export_csv():
    """
    导出csv
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")

    wxid = request.json.get("wxid")
    # st_ed_time = request.json.get("datetime", [0, 0])
    if not wxid:
        return ReJson(1002, body=f"username is required: {wxid}")
    # if not isinstance(st_ed_time, list) or len(st_ed_time) != 2:
    #     return ReJson(1002, body=f"datetime is required: {st_ed_time}")
    # start, end = st_ed_time
    # if not isinstance(start, int) or not isinstance(end, int) or start >= end:
    #     return ReJson(1002, body=f"datetime is required: {st_ed_time}")

    outpath = os.path.join(g.work_path, "export", my_wxid, "csv", wxid)
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, ret = export_csv(wxid, outpath, db_config)
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


@rs_api.route('/api/rs/export_json', methods=["GET", 'POST'])
def get_export_json():
    """
    导出json
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    db_config = get_conf(g.caf, my_wxid, "db_config")

    wxid = request.json.get("wxid")
    if not wxid:
        return ReJson(1002, body=f"username is required: {wxid}")

    outpath = os.path.join(g.work_path, "export", my_wxid, "json", wxid)
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, ret = export_json(wxid, outpath, db_config)
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


# end 导出聊天记录 *******************************************************************************************************

# start 聊天记录分析api **************************************************************************************************

@rs_api.route('/api/rs/date_count', methods=["GET", 'POST'])
@error9999
def get_date_count():
    """
    获取日期统计
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = get_conf(g.caf, my_wxid, "merge_path")
    date_count = DBHandler(merge_path).date_count()
    return ReJson(0, date_count)


@rs_api.route('/api/rs/wordcloud', methods=["GET", 'POST'])
@error9999
def wordcloud():
    pass


# 关于、帮助、设置 *******************************************************************************************************
@rs_api.route('/api/rs/check_update', methods=["GET", 'POST'])
@error9999
def check_update():
    """
    检查更新
    :return:
    """
    url = "https://api.github.com/repos/xaoyaoo/PyWxDump/tags"
    try:
        import requests
        res = requests.get(url)
        if res.status_code == 200:
            data = res.json()
            NEW_VERSION = data[0].get("name")
            if NEW_VERSION[1:] != pywxdump.__version__:
                msg = "有新版本"
            else:
                msg = "已经是最新版本"
            return ReJson(0, body={"msg": msg, "latest_version": NEW_VERSION,
                                   "latest_url": "https://github.com/xaoyaoo/PyWxDump/releases/tag/" + NEW_VERSION})
        else:
            return ReJson(2001, body="status_code is not 200")
    except Exception as e:
        return ReJson(9999, msg=str(e))


@rs_api.route('/api/rs/version', methods=["GET", 'POST'])
@error9999
def version():
    """
    版本
    :return:
    """
    return ReJson(0, pywxdump.__version__)


# END 关于、帮助、设置 ***************************************************************************************************


@rs_api.route('/')
@error9999
def index():
    return render_template('index.html')



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         local_server.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/08/01
# -------------------------------------------------------------------------------
import base64
import json
import logging
import os
import re
import time
import shutil
import pythoncom
import pywxdump

from flask import Flask, request, render_template, g, Blueprint, send_file, make_response, session
from pywxdump import get_core_db, all_merge_real_time_db, get_wx_db
from pywxdump.api.rjson import ReJson, RqJson
from pywxdump.api.utils import get_conf, get_conf_wxids, set_conf, error9999, gen_base64, validate_title, \
    get_conf_local_wxid, ls_loger
from pywxdump import get_wx_info, WX_OFFS, batch_decrypt, BiasAddr, merge_db, decrypt_merge, merge_real_time_db

from pywxdump.db import DBHandler, download_file, export_csv, export_json

ls_api = Blueprint('ls_api', __name__, template_folder='../ui/web', static_folder='../ui/web/assets/', )
ls_api.debug = False


# 以下为初始化相关 *******************************************************************************************************

@ls_api.route('/api/ls/init_last_local_wxid', methods=["GET", 'POST'])
@error9999
def init_last_local_wxid():
    """
    初始化，包括key
    :return:
    """
    local_wxid = get_conf_local_wxid(g.caf)
    local_wxid.remove(g.at)
    if local_wxid:
        return ReJson(0, {"local_wxids": local_wxid})
    return ReJson(0, {"local_wxids": []})


@ls_api.route('/api/ls/init_last', methods=["GET", 'POST'])
@error9999
def init_last():
    """
    是否初始化
    :return:
    """
    my_wxid = request.json.get("my_wxid", "")
    my_wxid = my_wxid.strip().strip("'").strip('"') if isinstance(my_wxid, str) else ""
    if not my_wxid:
        my_wxid = get_conf(g.caf, "auto_setting", "last")
    if my_wxid:
        set_conf(g.caf, "auto_setting", "last", my_wxid)
        merge_path = get_conf(g.caf, my_wxid, "merge_path")
        wx_path = get_conf(g.caf, my_wxid, "wx_path")
        key = get_conf(g.caf, my_wxid, "key")
        rdata = {
            "merge_path": merge_path,
            "wx_path": wx_path,
            "key": key,
            "my_wxid": my_wxid,
            "is_init": True,
        }
        if merge_path and wx_path:
            return ReJson(0, rdata)
    return ReJson(0, {"is_init": False, "my_wxid": ""})


@ls_api.route('/api/ls/init_key', methods=["GET", 'POST'])
@error9999
def init_key():
    """
    初始化，包括key
    :return:
    """
    wx_path = request.json.get("wx_path", "").strip().strip("'").strip('"')
    key = request.json.get("key", "").strip().strip("'").strip('"')
    my_wxid = request.json.get("my_wxid", "").strip().strip("'").strip('"')
    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")
    if not key:
        return ReJson(1002, body=f"key is required: {key}")
    if not my_wxid:
        return ReJson(1002, body=f"my_wxid is required: {my_wxid}")

    # db_config = get_conf(g.caf, my_wxid, "db_config")
    # if isinstance(db_config, dict) and db_config and os.path.exists(db_config.get("path")):
    #     pmsg = DBHandler(db_config)
    #     # pmsg.close_all_connection()

    out_path = os.path.join(g.work_path, "decrypted", my_wxid) if my_wxid else os.path.join(g.work_path, "decrypted")
    # 检查文件夹中文件是否被占用
    if os.path.exists(out_path):
        try:
            shutil.rmtree(out_path)
        except PermissionError as e:
            # 显示堆栈信息
            ls_loger.error(f"{e}", exc_info=True)
            return ReJson(2001, body=str(e))

    code, merge_save_path = decrypt_merge(wx_path=wx_path, key=key, outpath=str(out_path))
    time.sleep(1)
    if code:
        # 移动merge_save_path到g.work_path/my_wxid
        if not os.path.exists(os.path.join(g.work_path, my_wxid)):
            os.makedirs(os.path.join(g.work_path, my_wxid))
        merge_save_path_new = os.path.join(g.work_path, my_wxid, "merge_all.db")
        shutil.move(merge_save_path, str(merge_save_path_new))

        # 删除out_path
        if os.path.exists(out_path):
            try:
                shutil.rmtree(out_path)
            except PermissionError as e:
                # 显示堆栈信息
                ls_loger.error(f"{e}", exc_info=True)
        db_config = {
            "key": "merge_all",
            "type": "sqlite",
            "path": merge_save_path_new
        }
        set_conf(g.caf, my_wxid, "db_config", db_config)
        set_conf(g.caf, my_wxid, "merge_path", merge_save_path_new)
        set_conf(g.caf, my_wxid, "wx_path", wx_path)
        set_conf(g.caf, my_wxid, "key", key)
        set_conf(g.caf, my_wxid, "my_wxid", my_wxid)
        set_conf(g.caf, "auto_setting", "last", my_wxid)
        rdata = {
            "merge_path": merge_save_path_new,
            "wx_path": wx_path,
            "key": key,
            "my_wxid": my_wxid,
            "is_init": True,
        }
        return ReJson(0, rdata)
    else:
        return ReJson(2001, body=merge_save_path)


@ls_api.route('/api/ls/init_nokey', methods=["GET", 'POST'])
@error9999
def init_nokey():
    """
    初始化，包括key
    :return:
    """
    merge_path = request.json.get("merge_path", "").strip().strip("'").strip('"')
    wx_path = request.json.get("wx_path", "").strip().strip("'").strip('"')
    my_wxid = request.json.get("my_wxid", "").strip().strip("'").strip('"')

    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")
    if not merge_path:
        return ReJson(1002, body=f"merge_path is required: {merge_path}")
    if not my_wxid:
        return ReJson(1002, body=f"my_wxid is required: {my_wxid}")

    key = get_conf(g.caf, my_wxid, "key")

    set_conf(g.caf, my_wxid, "merge_path", merge_path)
    set_conf(g.caf, my_wxid, "wx_path", wx_path)
    set_conf(g.caf, my_wxid, "key", key)
    set_conf(g.caf, my_wxid, "my_wxid", my_wxid)
    set_conf(g.caf, "test", "last", my_wxid)
    rdata = {
        "merge_path": merge_path,
        "wx_path": wx_path,
        "key": "",
        "my_wxid": my_wxid,
        "is_init": True,
    }
    return ReJson(0, rdata)


# END 以上为初始化相关 ***************************************************************************************************


@ls_api.route('/api/ls/realtimemsg', methods=["GET", "POST"])
@error9999
def get_real_time_msg():
    """
    获取实时消息 使用 merge_real_time_db()函数
    :return:
    """
    my_wxid = get_conf(g.caf, g.at, "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    merge_path = get_conf(g.caf, my_wxid, "merge_path")
    key = get_conf(g.caf, my_wxid, "key")
    wx_path = get_conf(g.caf, my_wxid, "wx_path")

    if not merge_path or not key or not wx_path or not wx_path:
        return ReJson(1002, body="msg_path or media_path or wx_path or key is required")

    code, ret = all_merge_real_time_db(key=key, wx_path=wx_path, merge_path=merge_path)
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


# start 这部分为专业工具的api *********************************************************************************************

@ls_api.route('/api/ls/wxinfo', methods=["GET", 'POST'])
@error9999
def get_wxinfo():
    """
    获取微信信息
    :return:
    """
    import pythoncom
    pythoncom.CoInitialize()
    wxinfos = get_wx_info(WX_OFFS)
    pythoncom.CoUninitialize()
    return ReJson(0, wxinfos)


@ls_api.route('/api/ls/biasaddr', methods=["GET", 'POST'])
@error9999
def biasaddr():
    """
    BiasAddr
    :return:
    """
    mobile = request.json.get("mobile")
    name = request.json.get("name")
    account = request.json.get("account")
    key = request.json.get("key", "")
    wxdbPath = request.json.get("wxdbPath", "")
    if not mobile or not name or not account:
        return ReJson(1002)
    pythoncom.CoInitialize()
    rdata = BiasAddr(account, mobile, name, key, wxdbPath).run()
    return ReJson(0, str(rdata))


@ls_api.route('/api/ls/decrypt', methods=["GET", 'POST'])
@error9999
def decrypt():
    """
    解密
    :return:
    """
    key = request.json.get("key")
    if not key:
        return ReJson(1002)
    wxdb_path = request.json.get("wxdbPath")
    if not wxdb_path:
        return ReJson(1002)
    out_path = request.json.get("outPath")
    if not out_path:
        out_path = g.tmp_path
    wxinfos = batch_decrypt(key, wxdb_path, out_path=out_path)
    return ReJson(0, str(wxinfos))


@ls_api.route('/api/ls/merge', methods=["GET", 'POST'])
@error9999
def merge():
    """
    合并
    :return:
    """
    wxdb_path = request.json.get("dbPath")
    if not wxdb_path:
        return ReJson(1002)
    out_path = request.json.get("outPath")
    if not out_path:
        return ReJson(1002)
    db_path = get_wx_db(wxdb_path)
    # for i in db_path:print(i)
    rdata = merge_db(db_path, out_path)
    return ReJson(0, str(rdata))

# END 这部分为专业工具的api ***********************************************************************************************



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         parsingPublicMsg.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/07/03
# -------------------------------------------------------------------------------

# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         parsingMSG.py
# Description:
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
import json
import os
import re
from typing import Union, Tuple

import pandas as pd

from .dbbase import DatabaseBase
from .dbMSG import MsgHandler
from .utils import get_md5, name2typeid, typeid2name, type_converter, timestamp2str, xml2dict, match_BytesExtra, \
    db_error
import lz4.block
import blackboxprotobuf


class PublicMsgHandler(MsgHandler):
    _class_name = "PublicMSG"
    PublicMSG_required_tables = ["PublicMsg"]

    @db_error
    def PublicMSG_tables_exist(self):
        """
        判断该类所需要的表是否存在
        """
        return self.check_tables_exist(self.PublicMSG_required_tables)

    def PublicMsg_add_index(self):
        """
        添加索引,加快查询速度
        """
        # 检查是否存在索引
        sql = "CREATE INDEX IF NOT EXISTS idx_PublicMsg_StrTalker ON MSG(StrTalker);"
        self.execute(sql)
        sql = "CREATE INDEX IF NOT EXISTS idx_PublicMsg_CreateTime ON MSG(CreateTime);"
        self.execute(sql)
        sql = "CREATE INDEX IF NOT EXISTS idx_PublicMsg_StrTalker_CreateTime ON MSG(StrTalker, CreateTime);"
        self.execute(sql)

    @db_error
    def get_plc_msg_count(self, wxids: list = ""):
        """
        获取聊天记录数量,根据wxid获取单个联系人的聊天记录数量，不传wxid则获取所有联系人的聊天记录数量
        :param wxids: wxid list
        :return: 聊天记录数量列表 {wxid: chat_count}
        """
        if isinstance(wxids, str):
            wxids = [wxids]
        if wxids:
            wxids = "('" + "','".join(wxids) + "')"
            sql = f"SELECT StrTalker, COUNT(*) FROM PublicMsg WHERE StrTalker IN {wxids} GROUP BY StrTalker ORDER BY COUNT(*) DESC;"
        else:
            sql = f"SELECT StrTalker, COUNT(*) FROM PublicMsg GROUP BY StrTalker ORDER BY COUNT(*) DESC;"
        sql_total = f"SELECT COUNT(*) FROM MSG;"

        result = self.execute(sql)
        total_ret = self.execute(sql_total)

        if not result:
            return {}
        total = 0
        if total_ret and len(total_ret) > 0:
            total = total_ret[0][0]

        msg_count = {"total": total}
        msg_count.update({row[0]: row[1] for row in result})
        return msg_count

    @db_error
    def get_plc_msg_list(self, wxid="", start_index=0, page_size=500, msg_type: str = "", msg_sub_type: str = "",
                         start_createtime=None, end_createtime=None):
        sql_base = ("SELECT localId,TalkerId,MsgSvrID,Type,SubType,CreateTime,IsSender,Sequence,StatusEx,FlagEx,Status,"
                    "MsgSequence,StrContent,MsgServerSeq,StrTalker,DisplayContent,Reserved0,Reserved1,Reserved3,"
                    "Reserved4,Reserved5,Reserved6,CompressContent,BytesExtra,BytesTrans,Reserved2,"
                    "ROW_NUMBER() OVER (ORDER BY CreateTime ASC) AS id "
                    "FROM PublicMsg ")

        param = ()
        sql_wxid, param = ("AND StrTalker=? ", param + (wxid,)) if wxid else ("", param)
        sql_type, param = ("AND Type=? ", param + (msg_type,)) if msg_type else ("", param)
        sql_sub_type, param = ("AND SubType=? ", param + (msg_sub_type,)) if msg_type and msg_sub_type else ("", param)
        sql_start_createtime, param = ("AND CreateTime>=? ", param + (start_createtime,)) if start_createtime else (
            "", param)
        sql_end_createtime, param = ("AND CreateTime<=? ", param + (end_createtime,)) if end_createtime else ("", param)

        sql = (
            f"{sql_base} WHERE 1=1 "
            f"{sql_wxid}"
            f"{sql_type}"
            f"{sql_sub_type}"
            f"{sql_start_createtime}"
            f"{sql_end_createtime}"
            f"ORDER BY CreateTime ASC LIMIT ?,?"
        )
        param = param + (start_index, page_size)
        result = self.execute(sql, param)
        if not result:
            return [], []

        result_data = (self.get_msg_detail(row) for row in result)
        rdata = list(result_data)  # 转为列表
        wxid_list = {d['talker'] for d in rdata}  # 创建一个无重复的 wxid 列表

        return rdata, list(wxid_list)



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         server.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/01/04
# -------------------------------------------------------------------------------
import os
import subprocess
import sys
import time
import logging

server_loger = logging.getLogger("server")


def is_port_in_use(_host, _port):
    import socket
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        try:
            s.bind((_host, _port))
        except socket.error:
            return True
    return False


def start_falsk(merge_path="", wx_path="", key="", my_wxid="", port=5000, online=False, debug=False,
                isopenBrowser=True, loger_handler=None):
    """
    启动flask
    :param merge_path:  合并后的数据库路径
    :param wx_path:  微信文件夹的路径（用于显示图片）
    :param key:  密钥
    :param my_wxid:  微信账号(本人微信id)
    :param port:  端口号
    :param online:  是否在线查看(局域网查看)
    :param debug:  是否开启debug模式
    :param isopenBrowser:  是否自动打开浏览器
    :return:
    """
    work_path = os.path.join(os.getcwd(), "wxdump_work")  # 临时文件夹,用于存放图片等
    if not os.path.exists(work_path):
        os.makedirs(work_path)
        server_loger.info(f"[+] 创建临时文件夹：{work_path}")
        print(f"[+] 创建临时文件夹：{work_path}")

    conf_auto_file = os.path.join(work_path, "conf_auto.json")  # 用于存放各种基础信息
    at = "auto_setting"

    from flask import Flask, g
    from flask_cors import CORS
    from pywxdump.api import rs_api, ls_api, get_conf, set_conf

    # 检查端口是否被占用
    if online:
        host = '0.0.0.0'
    else:
        host = "127.0.0.1"

    app = Flask(__name__, template_folder='./ui/web', static_folder='./ui/web/assets/', static_url_path='/assets/')
    with app.app_context():
        # 设置超时时间为 1000 秒
        app.config['TIMEOUT'] = 1000
        app.secret_key = 'secret_key'

        app.logger.setLevel(logging.WARNING)
        if loger_handler:
            app.logger.addHandler(loger_handler)
            # 获取 Werkzeug 的日志记录器
            werkzeug_logger = logging.getLogger('werkzeug')
            # 将自定义格式器应用到 Werkzeug 的日志记录器
            werkzeug_logger.addHandler(loger_handler)
            werkzeug_logger.setLevel(logging.DEBUG)

        CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)  # 允许所有域名跨域

        @app.after_request  # 请求后的处理 用于解决部分用户浏览器不支持flask以及vue的js文件返回问题
        def changeHeader(response):
            disposition = response.get_wsgi_headers('environ').get(
                'Content-Disposition') or ''  # 获取返回头文件名描述，如'inline; filename=index.562b9b5a.js'
            if disposition.rfind('.js') == len(disposition) - 3:
                response.mimetype = 'application/javascript'
            return response

        @app.before_request
        def before_request():
            g.work_path = work_path  # 临时文件夹,用于存放图片等-新版本
            g.caf = conf_auto_file  # 用于存放各种基础信息-新版本
            g.at = at  # 用于默认设置-新版本

        if merge_path:
            set_conf(conf_auto_file, at, "merge_path", merge_path)
            db_config = {
                "key": "merge_all",
                "type": "sqlite",
                "path": "D:\\_code\\py_code\\pywxdumpProject\\z_test\\wxdump_work\\wxid_zh12s67kxsqs22\\merge_all.db"
            }
            set_conf(conf_auto_file, at, "db_config", db_config)
        if wx_path: set_conf(conf_auto_file, at, "wx_path", wx_path)
        if key: set_conf(conf_auto_file, at, "key", key)
        if my_wxid: set_conf(conf_auto_file, at, "my_wxid", my_wxid)
        if not os.path.exists(conf_auto_file):
            set_conf(conf_auto_file, at, "last", my_wxid)

        app.register_blueprint(rs_api)
        app.register_blueprint(ls_api)

        if isopenBrowser:
            try:
                # 自动打开浏览器
                url = f"http://127.0.0.1:{port}/"
                # 根据操作系统使用不同的命令打开默认浏览器
                if sys.platform.startswith('darwin'):  # macOS
                    subprocess.call(['open', url])
                elif sys.platform.startswith('win'):  # Windows
                    subprocess.call(['start', url], shell=True)
                elif sys.platform.startswith('linux'):  # Linux
                    subprocess.call(['xdg-open', url])
                else:
                    server_loger.error(f"Unsupported platform, can't open browser automatically.", exc_info=True)
                    print("Unsupported platform, can't open browser automatically.")
            except Exception as e:
                server_loger.error(f"自动打开浏览器失败：{e}", exc_info=True)

        if is_port_in_use(host, port):
            server_loger.error(f"Port {port} is already in use. Choose a different port.")
            print(f"Port {port} is already in use. Choose a different port.")
            input("Press Enter to exit...")
        else:
            time.sleep(1)
            server_loger.info(f"启动flask服务，host:port：{host}:{port}")
            print("[+] 请使用浏览器访问 http://127.0.0.1:5000/ 查看聊天记录")
            app.run(host=host, port=port, debug=debug, threaded=False)


if __name__ == '__main__':
    merge_path = r"****.db"

    wx_path = r"****"
    my_wxid = "****"

    start_falsk(merge_path=merge_path, wx_path=wx_path, my_wxid=my_wxid,
                port=5000, online=False, debug=False, isopenBrowser=False)



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         __init__.py.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
import pandas as pd

from .utils import download_file, dat2img

from .dbFavorite import FavoriteHandler
from .dbMSG import MsgHandler
from .dbMicro import MicroHandler
from .dbMedia import MediaHandler
from .dbOpenIMContact import OpenIMContactHandler
from .dbPublicMsg import PublicMsgHandler
from .dbOpenIMMedia import OpenIMMediaHandler

from .export.exportCSV import export_csv
from .export.exportJSON import export_json


class DBHandler(MicroHandler, MediaHandler, OpenIMContactHandler, PublicMsgHandler, OpenIMMediaHandler,
                FavoriteHandler):
    _class_name = "DBHandler"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.MSG_exist = self.Msg_tables_exist()
        self.Micro_exist = self.Micro_tables_exist()
        self.Media_exist = self.Media_tables_exist()
        self.OpenIMContact_exist = self.OpenIMContact_tables_exist()
        self.PublicMsg_exist = self.PublicMSG_tables_exist()
        self.OpenIMMedia_exist = self.OpenIMMedia_tables_exist()
        self.Favorite_exist = self.Favorite_tables_exist()

        if self.MSG_exist:  # 添加索引
            self.Msg_add_index()
        if self.PublicMsg_exist:  # 添加索引
            self.PublicMsg_add_index()
        if self.Micro_exist:  # 添加索引
            self.Micro_add_index()

        # print(self.MSG_exist, self.Micro_exist, self.Media_exist, self.OpenIMContact_exist, self.PublicMsg_exist,
        #       self.OpenIMMedia_exist, self.Favorite_exist)

    def get_user(self, word=None, wxids=None, labels=None):
        """
        获取联系人列表
        """
        users = self.get_user_list(word=word, wxids=wxids, label_ids=labels)
        if self.OpenIMContact_exist:
            users.update(self.get_im_user_list(word=word, wxids=wxids))
        return users
